/*
Copyright (c) 2006. The Regents of Columbia University, All Rights Reserved.
 
Permission to use, copy and distribute this software and its documentation 
for any purpose, without fee, and without a written agreement, is hereby granted, 
provided that the above copyright notice, this paragraph and the following two 
paragraphs appear in all copies and distributions.  

Created by Lulin Song, 
           International Research Institute Society,
           Columbia University

                       
REGENTS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A  PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION,
IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS". REGENTS HAS
NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
MODIFICATIONS.
*/

#ifndef WIN_EXTRA_H
#define WIN_EXTRA_H

/* Define some routines or macros which don't exist in Windows.
 *  @author Lulin Song (International Research Institute For Climate Society, Columbia University)
 */

#ifdef WIN32
#include <float.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#define isnan(x) _isnan(x)
#define finite(x) _finite(x)
#define strcpy(x,y) strcpy_s(x,y) //strcpy is depreciated on Windows

//#define drand48() drand16() //16 bits precision

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

/** Define some constants from book "Numerical Recipes in Fortran 90" Pg. 1142 */
#define IA 16807
#define IM 2147483647
#define IQ 127773
#define IR 2836

#ifdef  __cplusplus
extern "C" {
#endif

typedef long long int uint64_t;
typedef long int uint32_t;

/** drand48_data is copied from GNU stdlib.h from Linux */
/* Data structure for communication with thread safe versions.  This
   type is to be regarded as opaque.  It's only exported because users
   have to allocate objects of this type.  */
struct drand48_data
  {
    unsigned short int __x[3];	/* Current state.  */
    unsigned short int __old_x[3]; /* Old state.  */
    unsigned short int __c;	/* Additive const. in congruential formula.  */
    unsigned short int __init;	/* Flag for initializing.  */
    unsigned long long int __a;	/* Factor in congruential formula.  */
  };

union ieee754_double
  {
    double d;

    /* This is the IEEE 754 double-precision format.  */
    struct
      {
//#if	__BYTE_ORDER == __BIG_ENDIAN
//	unsigned int negative:1;
//	unsigned int exponent:11;
	/* Together these comprise the mantissa.  */
//	unsigned int mantissa0:20;
//	unsigned int mantissa1:32;
//#endif				/* Big endian.  */
//#if	__BYTE_ORDER == __LITTLE_ENDIAN
//# if	__FLOAT_WORD_ORDER == BIG_ENDIAN
//	unsigned int mantissa0:20;
//	unsigned int exponent:11;
//	unsigned int negative:1;
//	unsigned int mantissa1:32;
//# else
	/* Together these comprise the mantissa.  */
	unsigned int mantissa1:32;
	unsigned int mantissa0:20;
	unsigned int exponent:11;
	unsigned int negative:1;
//# endif
//#endif				/* Little endian.  */
      } ieee;
};

#define IEEE754_DOUBLE_BIAS	0x3ff /* Added to exponent.  */

int
__drand48_iterate (unsigned short int xsubi[3],struct drand48_data *buffer);

int
__erand48_r (unsigned short int xsubi[3], struct drand48_data *buffer, double *result);

/* Global state for non-reentrant functions.  */
//struct drand48_data __libc_drand48_data;

/** functions */
double (__cdecl drand48 ) ();
void srand48(time_t x);

double randomNumberGenerator(long int idum);

void getWindowFileName( char* filename);

#ifdef __cplusplus
}
#endif

#endif

#ifdef CYGWIN
void getWindowFileName( char* filename);
#endif

#endif
